/***************************************************************************
 *   Copyright (C) 2007-2009 by Glad Deschrijver                           *
 *   glad.deschrijver@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/

#ifndef QONJUGATOR_PLUGIN_H
#define QONJUGATOR_PLUGIN_H

#include <QObject>
#include <QStringList>
#include "qonjugatortense.h"

/*!
 * \mainpage Qonjugator
 * \author Glad Deschrijver
 *
 * This is Qonjugator, a program which conjugates verbs in different languages.
 */

namespace Qonjugator
{

/*!
 * \interface Plugin plugin.h <qonjugator/plugin.h>
 * \author Glad Deschrijver
 *
 * All plugins should extend this interface.
 *
 * Plugins for Qonjugator provide the conjugation of the verbs in a
 * certain language (e.g. English, Spanish).  The plugins are listed
 * in the menu "Settings" and the correct plugin is selected by the
 * user before actually conjugating verbs.
 *
 * This interface provides basic functionality which may be extended
 * by each plugin in order to give the most appropriate results.
 */

class Plugin : public QObject
{
public:
	virtual ~Plugin() {}

	/*!
	 * This function generates a Verb structure which consists of
	 * the infinitive and a list of the different tenses.
	 * The tenses are stored with the name of the mood of the
	 * tense (e.g. indicative, subjunctive) and the name of the
	 * tense (e.g. present, past).  The tenses must be ordered
	 * in the list in the same order as you wish them to appear
	 * on screen.
	 *
	 * Note that this function should already do the complete
	 * conjugation and store it in the Verb structure.
	 *
	 * \param verb a string containing the infinitive of the verb
	 * \return a Verb structure containing the conjugation
	 */

	virtual Verb* createVerb(const QString &verb) = 0;

	/*!
	 * This function returns a description of the current plugin
	 * describing the language for which it conjugates and its usage.
	 * \return a string containing the description of the plugin
	 */

	virtual QString description() const
	{
		return QString("<h1>") + language() + "</h1>"
		    + tr("<p>This is a %1 verb conjugator.</p>"
		    "<p>Enter your verb in the text field above and "
		    "press \"Conjugate\".").arg(language());
	}

	/*!
	 * This function returns a "What's This?" description of the
	 * current plugin describing the language for which it
	 * conjugates and its usage.
	 * \return a string containing the description of the plugin
	 */

	virtual QString inputInstructions() const
	{
		return QObject::tr("Enter a verb and press \"Conjugate\".");
	}

	/*!
	 * This function returns the language in which the current
	 * plugin conjugates the verbs (e.g. Spanish).
	 * \return a string describing the language of the plugin
	 */

	virtual QString language() const
	{
		return tr("Language");
	}

	/*!
	 * This function returns a list of special characters that
	 * may occur in infinitives of the language in which the current
	 * plugin conjugates the verbs.
	 * \return a QStringList containing the special characters for this plugin
	 */

	virtual QStringList specialCharacters() const
	{
		QStringList emptyList;
		return emptyList;
	}

	/*!
	 * This function tests whether the plugin is capable to conjugate the
	 * given verb.
	 * \param verb the infinitive of the verb
	 * \return true if the verb can be conjugated using this plugin, false otherwise
	 */

	virtual bool canConjugate(const QString &verb) const
	{
		Q_UNUSED(verb);
		return true;
	}

	/*!
	 * This function returns the infinitive of the current verb
	 * in a human readable form (i.e. in UTF-8 encoding; also any
	 * accented characters should be in UTF-8 encoding).
	 * \param verb the structure containing the conjugation of the current verb
	 * \return a string containing the infinitive
	 */

	virtual QString infinitive(Verb *verb) const
	{
		return verb->infinitive;
	}

	/*!
	 * This function returns the base infinitive of the current verb
	 * in a human readable form (i.e. in UTF-8 encoding; also any
	 * accented characters should be in UTF-8 encoding).  If the
	 * verb is reciprocal, then the base infinitive is formed by
	 * removing the reciprocal pronoun, for non-reciprocal verbs
	 * the base infinitive is the same as the infinitive.  E.g.
	 * in French, the base infinitive of "se battre" is equal to
	 * "battre", which is also the base infinitive of "battre".
	 * \param verb the structure containing the conjugation of the current verb
	 * \return a string containing the base infinitive
	 */

	virtual QString infinitiveBase(Verb *verb) const
	{
		if (verb->infinitiveBase.isEmpty())
			verb->infinitiveBase = verb->infinitive;
		return verb->infinitiveBase;
	}

	/*!
	 * This function returns the name of the mood of the
	 * tenseNum-th tense of the current verb (e.g. "Indicative",
	 * "Subjunctive").
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \return a string containing the name of the mood of the tenseNum-th tense
	 */

	virtual QString tenseMood(Verb *verb, int tenseNum) const
	{
		return verb->tenses.at(tenseNum)->mood;
	}

	/*!
	 * This function returns the name of the tenseNum-th tense of
	 * the current verb (e.g. "present", "past").
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \return a string containing the name of the tenseNum-th tense
	 */

	virtual QString tenseName(Verb *verb, int tenseNum) const
	{
		return verb->tenses.at(tenseNum)->name;
	}

	/*!
	 * This function returns the subject pronoun corresponding to
	 * the timeNum-th time of the tenseNum-th tense of the current
	 * verb (e.g. the subject pronoun corresponding to the first
	 * time of the first tense (present) is "I", the subject
	 * pronoun for the second time of the second tense (past) is
	 * "you").  This function returns a QString containing the
	 * substring %1 which will be replaced by the corresponding
	 * form in the conjugation.  E.g. in French, getSubjectPronoun
	 * will return for the first person plural of the imperative
	 * "%1-nous", for the verb "s'entendre" the string %1 will
	 * be replaced in the conjugation by "entendons", so in
	 * the conjugation we will obtain "entendons-nous".
	 * \param tenseNum the number of the tense
	 * \param timeNum the number of the time
	 * \return a string containing the subject pronoun and a substring %1 to be replaced by the actual form of the conjugation
	 */

	virtual QString subjectPronoun(Verb *verb, int tenseNum, int timeNum) const
	{
		Q_UNUSED(verb);
		Q_UNUSED(tenseNum);
		Q_UNUSED(timeNum);
		return "";
	}

	/*!
	 * This function returns the timeNum-th time of the tenseNum-th
	 * tense of the current verb in UTF-8 encoding (e.g. the first time
	 * of the first tense (present) of "to be" is "am", the second time
	 * of the first tense is "are", the first time of the second tense
	 * (past) is "was").
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \param timeNum the number of the time
	 * \return a string containing the timeNum-th time of the tenseNum-th tense
	 */

	virtual QString tense(Verb *verb, int tenseNum, int timeNum) const
	{
		return verb->tenses.at(tenseNum)->forms.at(timeNum);
	}

	/*!
	 * This function returns a list of the irregularity types that may
	 * exist in the conjugation of the verbs in the current language.
	 * \return a list of strings containing the description of the different irregularity types
	 */

	virtual QStringList irregularityTypeNames() const
	{
		return QStringList();
	}

	/*!
	 * This function returns a list of the irregularity type classes
	 * corresponding to the irregularity types given by irregularityTypeNames().
	 * \return a list of integers describing the irregularity type classes
	 */

	virtual QList<int> defaultIrregularityTypeClasses() const
	{
		return QList<int>();
	}

	/*!
	 * This function returns the irregularity type of the type-th irregular
	 * form, the function getIrregularityTypeNumber() can be used to obtain
	 * the number of the irregularity type of a form (e.g. in Spanish, if
	 * the irregular form "duermo" of the verb "dormir" is of type 2, then
	 * getIrregularityTypeName(2) may return "o->ue conversion").
	 * \param verb the structure containing the conjugation of the current verb
	 * \param type the number of the irregularity type
	 * \return a string containing the description of the type-th irregularity type
	 */

	virtual QString irregularityTypeName(int) const
	{
		return QString();
	}

	/*!
	 * This function returns the number of the irregularity type of
	 * the timeNum-th time of the tenseNum-th tense of the current
	 * verb (e.g. in Spanish, if the irregularity type of the first
	 * time of the first tense (indicativo presente) of the verb
	 * "dormir" ("duermo") is "o->ue conversion" has been
	 * given the number 2, then getIrregularityTypeNumber(verb, 1, 1)
	 * will return 2).
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \param timeNum the number of the time
	 * \return the number of the irregularity type of the timeNum-th time of the tenseNum-th tense
	 */

	virtual int irregularityTypeNumber(Verb *verb, int tenseNum, int timeNum) const
	{
		return verb->tenses.at(tenseNum)->irregularityTypes.at(timeNum);
	}

	/*!
	 * This function returns the number of irregularity types provided for
	 * the current language where the regular verbs constitute one type (e.g.
	 * the Spanish language there are 13 irregularity types including the
	 * regular verbs).
	 * \return the number of irregularity types
	 */

	virtual int numberOfIrregularityTypes() const
	{
		return 1;
	}

	/*!
	 * This function returns the total number of tenses
	 * (disregarding to which mood they belong).
	 * \param verb the structure containing the conjugation of the current verb
	 * \return an integer containing the total number of tenses
	 */

	virtual int numberOfTenses(Verb *verb) const
	{
		return verb->tenses.size();
	}

	/*!
	 * This function returns the total number of times in the
	 * tenseNum-th tense (e.g. the first tense (indicative present)
	 * contains 6 times).
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \return an integer containing the number of times in the tenseNum-th tense
	 */

	virtual int numberOfForms(Verb *verb, int tenseNum) const
	{
		return verb->tenses.at(tenseNum)->forms.size();
	}

	/*!
	 * This function returns a list of verbs which have a similar
	 * conjugation as the input verb.
	 * \param verb the structure containing the conjugation of the current verb
	 * \return a QStringList containing the infinitives of the similarly conjugated verbs
	 */

	virtual QStringList similarVerbs(Verb*) const
	{
		QStringList emptyList;
		return emptyList;
	}

signals:
	/*!
	 * This signal is emitted whenever an error occurs during the
	 * conjugation of the verb.  The message argument specifies the
	 * details of the error.
	 * \param message the message describing the error
	 */
	void showError(const QString &message);

/*!
 * The protected members of this class are not necessary and do not
 * have to be (re)implemented.  They are provided for the convenience
 * of the programmer and may be useful to implement createVerbs().
 */
protected:
	/*!
	 * This function appends a tense to the list of tenses in verb.
	 * The mood and the name of this tense are defined as tenseMood
	 * and tenseName respectively.  The number of times that this
	 * tense may contain is specified by numberOfForms.
	 *
	 * Note that this function does not generate the actual
	 * conjugation of the tense, only the structure which will
	 * contain the tense is created.
	 * \param verb the structure which will contain the conjugation of the current verb
	 * \param tenseMood the name of the mood of this tense (e.g. "Indicative")
	 * \param tenseName the name of this tense (e.g. "Present")
	 * \param numberOfForms the number of times that this tense will contain (e.g. 6)
	 */

	virtual void appendTense(Verb *verb, const QString &tenseMood, const QString &tenseName, int numberOfForms)
	{
		Tense *tensetmp = new Tense;
		tensetmp->mood = tenseMood;
		tensetmp->name = tenseName;
		for (int i = 0; i < numberOfForms; ++i)
		{
			tensetmp->forms.append("");
			tensetmp->irregularityTypes.append(0);
		}
		verb->tenses.append(tensetmp);
	}

	/*!
	 * This is an overloaded member function, provided for convenience.
	 * This function appends a tense to the list of tenses in verb.
	 * The name of this tense is defined as tenseName, the name of
	 * the mood is empty.  The number of times that this
	 * tense may contain is specified by numberOfForms.
	 *
	 * Note that this function does not generate the actual
	 * conjugation of the tense, only the structure which will
	 * contain the tense is created.
	 * \param verb the structure which will contain the conjugation of the current verb
	 * \param tenseName the name of this tense (e.g. "Present")
	 * \param numberOfForms the number of times that this tense will contain (e.g. 6)
	 */

	virtual void appendTense(Verb *verb, const QString &tenseName, int numberOfForms)
	{
		appendTense(verb, "", tenseName, numberOfForms);
	}

	/*!
	 * This function sets the timeNum-th time of the tenseNum-th
	 * tense of the current verb (e.g. the value of the first time
	 * of the first tense (present) of "to be" is "am", the value of
	 * the second time of the first tense is "are", the value of the
	 * first time of the second tense (past) is "was").
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \param timeNum the number of the time
	 * \param value a string containing the timeNum-th time of the tenseNum-th tense
	 */

	virtual void setTense(Verb *verb, int tenseNum, int timeNum, const QString &value)
	{
		verb->tenses.at(tenseNum)->forms.replace(timeNum, value);
	}

	/*!
	 * This function sets the number of the irregularity type of
	 * the timeNum-th time of the tenseNum-th tense of the current
	 * verb (e.g. in Spanish, the irregularity type of the first
	 * time of the first tense (indicativo presente) of the verb
	 * "dormir" ("duermo") is "o->ue conversion" which can be
	 * given the number 1).
	 * \param verb the structure containing the conjugation of the current verb
	 * \param tenseNum the number of the tense
	 * \param timeNum the number of the time
	 * \param type the number of the irregularity type of the timeNum-th time of the tenseNum-th tense
	 */

	virtual void setIrregularityTypeNumber(Verb *verb, int tenseNum, int timeNum, int type)
	{
		verb->tenses.at(tenseNum)->irregularityTypes.replace(timeNum, type);
	}
};

} // end namespace

Q_DECLARE_INTERFACE(Qonjugator::Plugin, "glad.Qonjugator.Plugin/1.0")

#endif
